package main

import (
	"errors"
	"fmt"
	"os"
	"time"

	"git.tcp.direct/kayos/prox5"
	"github.com/rs/zerolog"
)

// Config
var (
	// Round-robin proxy
	workers = 1000  // Workers (Speed)
	debug   = true  // Debug mode (Logs)
	recycle = true  // Recycle proxies
	redact  = false // Redact proxy address (Logs)
	remove  = 5     // Removal recylce timeout
	timeout = 5     // Validation timeout

	// Socks5 proxy
	proxy    = true             // Enable socks5 proxy
	address  = "127.0.0.1:9999" // Listener IP address & port
	username = "user"           // Username (Authentication)
	password = "pass"           // Password (Authentication)

	log Logger // Access logger
)

// Socks5 proxy logger
type Logger struct {
	Zero zerolog.Logger // Fancy logger
}

// Custom logger printf
func (l Logger) Printf(format string, a ...interface{}) {
	l.Zero.Debug().Msg(format)
}

// Custom logger print
func (l Logger) Print(str string) {
	l.Zero.Debug().Msg(str)
}

// Custom logger errorf
func (l Logger) Errorf(format string, a ...interface{}) {
	err := errors.New(format)
	l.Zero.Debug().Err(err)
}

func main() {
	// Create a logger
	log = Logger{
		Zero: zerolog.New(zerolog.ConsoleWriter{
			Out: os.Stdout,
		}).With().Timestamp().Logger(),
	}

	// Create a pool
	swamp := prox5.NewProxyEngine()

	// Use config
	swamp.SetMaxWorkers(workers)
	swamp.LoadProxyTXT("proxies.txt")
	swamp.SetRemoveAfter(remove)
	swamp.SetValidationTimeout(time.Second * time.Duration(timeout))

	// Recycle mode enabled
	if recycle {
		swamp.EnableRecycling()
	}

	// Debug mode enabled
	if debug {
		swamp.EnableDebug()
		swamp.SetDebugLogger(log)
		zerolog.SetGlobalLevel(zerolog.DebugLevel)
	}

	// Debug redact enabled
	if redact {
		swamp.EnableDebugRedaction()
	}

	// Start the pool
	err := swamp.Start()
	// Handle the error
	if err != nil {
		fmt.Printf("Error: %s", err.Error())
		os.Exit(1)
	}

	// Proxy mode enabled
	if proxy {
		// Start the round-robin proxy
		err = swamp.StartSOCKS5Server(address, username, password)

		// Handle the error
		if err != nil {
			fmt.Printf("Error: %s", err.Error())
			os.Exit(1)
		}
	}

	// Block forever
	for {
	}
}
